import React, { useState, useEffect } from "react";
import { css } from "@emotion/css";
import tw, { styled } from "twin.macro";
import PropTypes from "prop-types";

const Navigation = (props) => {
  const { metaData = null, navigationRef = null } = props;

  const [screenWidth, setScreenWidth] = useState(0);

  useEffect(() => {
    const handleScreen = () => {
      setScreenWidth(window.innerWidth);
    };

    window.addEventListener("resize", handleScreen);

    handleScreen();

    return () => {
      window.removeEventListener("resize", handleScreen);
    };
  }, [screenWidth]);

  if (!metaData) {
    return null;
  }

  const {
    templateAlignment = "left", // center, right
    hasDivider = false,
    logoName = "",
    logoImage = "",
    navMenuList = [],
    backgroundColor = "#95BAA8",
    textColor = "#FFFFFF",
    backgroundImage = {},
    backgroundOpacity = 1,
    isMobile = false,
  } = metaData;

  const StyleMenuList = styled.a(({ hasDivider }) => [
    `
    padding-right:16px;
    font-weight: 300;
    font-size: 14px;
    color: ${textColor};
    text-decoration: none;

    &:hover {
        text-decoration: underline;
    }
    
    ${
      isMobile
        ? ` display: none; `
        : `
        @media screen and (max-width: 900px) {
            display: none;
        }
      `
    }
  `,
    hasDivider &&
      `
        padding-left: 16px;
        border-right: 1px solid ${textColor};

        &:last-child{
        border-right: none;
        }
    `,

    templateAlignment == "center" &&
      `
        margin-top: 10px;
    `,
  ]);

  const styleHamburger = (width) => {
    return {
      display: "block",
      width: `${width ? width + "px" : "25.45px"}`,
      height: "1.5px",
      backgroundColor: textColor,
      borderRadius: "10px",
      margin: "5px 0px",
      transition: "all 0.3s ease-in-out",
    };
  };

  const navBackground = backgroundImage.image
    ? backgroundImage?.arrange == "tile"
      ? `url(${backgroundImage?.image}) 0% 0%/calc(100% / 8) repeat scroll`
      : backgroundImage?.arrange == "fill"
      ? `url(${backgroundImage?.image}) 0% 0%/100% repeat-y scroll`
      : backgroundImage?.arrange == "fix"
      ? `url(${backgroundImage?.image}) center center/cover no-repeat fixed `
      : backgroundColor
    : backgroundColor;

  const displayBackground =
    navBackground.substr(0, 1) == "#"
      ? backgroundColor
      : `${navBackground}, ${backgroundColor}`;

  return (
    // NAVBAR
    <div
      ref={navigationRef}
      className={css([
        `
          height: 50px;
          padding: 8px 20px;
          display: flex;
          align-items: center;
          justify-content: space-between;
          color: ${textColor};
          font-size: 16px;
          font-weight: 600;
          position: sticky;
          width: -webkit-fill-available
          left: 0;
          top: 0;
          z-index: 9999;
          border-bottom: 1px solid ${textColor};

          &:before {
            content: "";
            position: absolute;
            z-index: -1;
            top: 0;
            bottom: 0;
            left: 0;
            right: 0;
            background: ${displayBackground};
            opacity: ${backgroundOpacity};
          }
        `,

        templateAlignment == "center" &&
          `
          justify-content: center;
          flex-direction: ${
            navMenuList.length == 0 || screenWidth <= 900 ? "row" : "column"
          };
          height: ${
            navMenuList.length == 0 || screenWidth <= 900 ? "50px" : "90px"
          };


          ${
            isMobile
              ? `flex-direction: row;`
              : `
              @media (max-width: 900px) {
                flex-direction: row;
              }
            `
          }
        `,

        templateAlignment == "right" && {
          flexDirection: "row-reverse",
        },

        `
        ${
          isMobile
            ? ` column-gap: 24px;`
            : `
            @media (max-width: 900px) {
              column-gap: 24px;
            }
          `
        }
        `,
      ])}
    >
      {/* LOGO */}
      <div
        className={css([
          logoImage && {
            // height: "40px",
            // objectFit: "contain",
            display: "flex",
            alignItems: "center",
          },
          logoImage == "" &&
            `
          position: relative;
          text-overflow: ellipsis;
          overflow: hidden; 
          width: 100%; 
          text-align: ${
            templateAlignment == "center"
              ? "center"
              : templateAlignment == "right"
              ? "right"
              : "left"
          };
          white-space: nowrap;
          `,
        ])}
      >
        {logoImage ? (
          <img src={logoImage} height={50} style={{ padding: "8px 0px" }} />
        ) : logoName ? (
          <span>{logoName}</span>
        ) : null}
      </div>

      {/* MENU */}
      <div
        className={css([
          `
          display: flex;
          align-items: center;
          white-space: nowrap;

          ${
            isMobile
              ? `
            position: absolute;
            flex-direction: column;
            height: 0px;
            width: 100%;
            margin: auto;
            margin-top: 5px;
            transition: all 0.3s ease-out;
            top: 52px;
            left: 0;

            &::before {
              content: "";
              position: absolute;
              z-index: -1;
              top: 0;
              bottom: 0;
              left: 0;
              right: 0;
              background: ${displayBackground};
              opacity: ${backgroundOpacity};
            }
          `
              : `
          @media (max-width: 900px) {
            position: absolute;
            flex-direction: column;
            height: 0px;
            width: 100%;
            margin: auto;
            margin-top: 5px;
            transition: all 0.3s ease-out;
            top: 52px;
            left: 0;

            &::before {
              content: "";
              position: absolute;
              z-index: -1;
              top: 0;
              bottom: 0;
              left: 0;
              right: 0;
              background: ${displayBackground};
              opacity: ${backgroundOpacity};
            }
          }
          `
          }
          `,
        ])}
      >
        {navMenuList.length > 0
          ? navMenuList.map((item, index) => (
              <StyleMenuList key={index} hasDivider={hasDivider}>
                <span>
                  {screenWidth < 1300 &&
                  screenWidth > 900 &&
                  templateAlignment != "center" &&
                  item.title.length > 12 &&
                  navMenuList.length > 3
                    ? item?.title.substring(0, 12) + "..."
                    : item?.title}
                </span>
              </StyleMenuList>
            ))
          : null}
      </div>

      {/* hamburger */}
      {navMenuList.length == 0 ? null : (
        <>
          <div
            className={css([
              `
                display: none;

                ${
                  isMobile
                    ? `
                    display: block;
                    z-index: 99;
                  `
                    : `
                    @media screen and (max-width: 900px) {
                      display: block;
                      z-index: 99;
                    }
                  `
                }
              `,
              templateAlignment == "center" &&
                `

            ${
              isMobile
                ? `
                position: absolute;
                right: 1rem; `
                : `
                @media screen and (max-width: 900px) {
                  position: absolute;
                  right: 1rem;
                }
              `
            }
            `,
            ])}
          >
            <span className={css([styleHamburger()])}></span>

            <span className={css([styleHamburger("20.86")])}></span>

            <span className={css([styleHamburger()])}></span>
          </div>
        </>
      )}
    </div>
  );
};

Navigation.propTypes = {
  metaData: PropTypes.shape({
    templateAlignment: PropTypes.string,
    hasDivider: PropTypes.bool,
    logoImage: PropTypes.string,
    logoName: PropTypes.string,
    navMenuList: PropTypes.array,
    backgroundColor: PropTypes.string,
    textColor: PropTypes.string,
    backgroundImage: PropTypes.shape({
      arrange: PropTypes.oneOf(["fill", "tile", "fix"]),
      image: PropTypes.string,
    }),
    backgroundOpacity: PropTypes.number,
  }),
  navigationRef: PropTypes.func,
};

Navigation.defaultProps = {
  metaData: {
    templateAlignment: "",
    hasDivider: false,
    logoImage: "สั่งซื้อสินค้า",
    logoName: "#26D496",
    navMenuList: [],
    backgroundColor: "#95BAA8",
    textColor: "#FFFFFF",
    backgroundImage: {
      arrange: "",
      image: "",
    },
    backgroundOpacity: 1,
  },
  navigationRef: null,
};

export default Navigation;
